@@ -42,56 +42,65 @@ def main(args=None):
         level=getattr(logging, parsed_args.log_level.upper(), logging.INFO),
         format="%(asctime)s:%(levelname)s:%(message)s",
     )
-    embody_serial = EmbodySerial(serial_port=parsed_args.device)
+    try:
+        embody_serial = EmbodySerial(serial_port=parsed_args.device)
+    except Exception as e:
+        print(f"Connection error: {e}")
+        exit(-1)
+
     send_helper = EmbodySendHelper(sender=embody_serial)
     try:
         if parsed_args.get:
             print(f"{getattr(send_helper, get_attributes_dict.get(parsed_args.get))()}")
-            exit(0)
         elif parsed_args.get_all:
             __get_all_attributes(send_helper)
-            exit(0)
         elif parsed_args.set_time:
             print(f"Set current time: {send_helper.set_current_timestamp()}")
             print(f"New current time is: {send_helper.get_current_time()}")
-            exit(0)
         elif parsed_args.set_trace_level:
             print(
                 f"Trace level set: {send_helper.set_trace_level(parsed_args.set_trace_level)}"
             )
-            exit(0)
         elif parsed_args.list_files:
-            __list_files(send_helper)
-            exit(0)
+            try:
+                __list_files(send_helper)
+            except Exception as e:
+                print(f"Error listing files: {e}")
         elif parsed_args.download_file:
-            __download_file(parsed_args.download_file, embody_serial, send_helper)
-            exit(0)
+            try:
+                __download_file(parsed_args.download_file, embody_serial, send_helper)
+            except Exception as e:
+                print(f"Error downloading file: {e}")
         elif parsed_args.download_file_with_delay:
-            __download_file(
-                parsed_args.download_file_with_delay, embody_serial, send_helper, 0.01
-            )
-            exit(0)
+            try:
+                __download_file(
+                    parsed_args.download_file_with_delay,
+                    embody_serial,
+                    send_helper,
+                    0.01,
+                )
+            except Exception as e:
+                print(f"Error downloading file with delay: {e}")
         elif parsed_args.download_files:
-            __download_files(embody_serial, send_helper)
-            exit(0)
+            try:
+                __download_files(embody_serial, send_helper)
+            except Exception as e:
+                print(f"Error downloading files: {e}")
         elif parsed_args.delete_file:
             print(
                 f"Delete file {parsed_args.delete_file}:"
                 f" {send_helper.delete_file(file_name=parsed_args.delete_file)}"
             )
-            exit(0)
         elif parsed_args.delete_files:
             print(f"Delete files: {send_helper.delete_all_files()}")
-            exit(0)
         elif parsed_args.reformat_disk:
             print(f"Reformatting disk: {send_helper.reformat_disk()}")
-            exit(0)
         elif parsed_args.reset:
             print(f"Resetting device: {send_helper.reset_device()}")
-            exit(0)
         elif parsed_args.reboot:
             print(f"Rebooting device: {send_helper.reboot_device()}")
-            exit(0)
+        else:
+            pass
 
     finally:
         embody_serial.shutdown()
