@@ -5,6 +5,7 @@ and subscribing for incoming messages from the device.
 """
 import concurrent.futures
 import logging
+import re
 import struct
 import sys
 import tempfile
@@ -15,6 +16,7 @@ from abc import abstractmethod
 from concurrent.futures import ThreadPoolExecutor
 from concurrent.futures import TimeoutError
 from dataclasses import dataclass
+from operator import attrgetter
 from typing import Optional
 
 import serial
@@ -24,7 +26,6 @@ from embodycodec import crc
 from embodycodec import types
 from serial.serialutil import SerialBase
 from serial.serialutil import SerialException
-from serial.tools import list_ports_common
 
 from embodyserial.exceptions import CrcError
 from embodyserial.exceptions import MissingResponseError
@@ -102,7 +103,10 @@ class EmbodySerial(ConnectionListener, EmbodySender):
             if not self.__connected:
                 return
             self.__connected = False
-            self.__serial.close()
+            if self.__serial.is_open:
+                self.__serial.reset_input_buffer()
+                self.__serial.reset_output_buffer()
+                self.__serial.close()
             self.__reader.stop()
             self.__sender.shutdown()
 
@@ -128,30 +132,30 @@ class EmbodySerial(ConnectionListener, EmbodySender):
         """
         if size == 0:
             return tempfile.NamedTemporaryFile(delete=False).name
-        self.send_async(codec.GetFileUart(types.File(file_name)))
-        return self.__reader.download_file(
-            file_name, size, download_listener, timeout, delay
-        )
+
+        # lock send to prevent sending other messages while downloading
+        with self.__sender._send_lock:
+            return self.__reader.download_file(
+                file_name, size, download_listener, timeout, delay
+            )
 
     @staticmethod
     def __find_serial_port() -> str:
         """Find first matching serial port name."""
-        manufacturers = ["Datek", "Aidee"]
-        descriptions = ["IsenseU", "G3", "EmBody"]
         all_available_ports = serial.tools.list_ports.comports()
-        if len(all_available_ports) == 0:
+        if not all_available_ports:
             raise SerialException("No available serial ports")
+
+        all_available_ports.sort(key=attrgetter("device"))
         for port in all_available_ports:
-            candidate: Optional[list_ports_common.ListPortInfo] = None
-            if any(description in port.description for description in descriptions):
-                candidate = port
-            elif port.manufacturer and any(
-                manufacturer in port.manufacturer for manufacturer in manufacturers
+            if (
+                not re.search("Datek|Aidee", str(port.manufacturer))
+                and not re.search("IsenseU|G3|EmBody", str(port.description))
+                and sys.platform != "win32"
             ):
-                candidate = port
-            elif sys.platform == "win32":
-                candidate = port
-            if candidate and EmbodySerial.__port_is_alive(port):
+                continue
+
+            if EmbodySerial.__port_is_alive(port):
                 return port.device
         raise SerialException("No matching serial ports found")
 
@@ -160,7 +164,7 @@ class EmbodySerial(ConnectionListener, EmbodySender):
         """Check if port has an active embody device."""
         logging.info(f"Checking candidate port: {port}")
         try:
-            ser = serial.Serial(port=port.device, baudrate=115200, timeout=3)
+            ser = serial.Serial(port=port.device, baudrate=115200, timeout=1)
             ser.write(codec.Heartbeat().encode())
             expected_response = codec.HeartbeatResponse().encode()
             response = ser.read(len(expected_response))
@@ -179,7 +183,7 @@ class _MessageSender(ResponseMessageListener):
 
     def __init__(self, serial_instance: SerialBase) -> None:
         self.__serial = serial_instance
-        self.__send_lock = threading.Lock()
+        self._send_lock = threading.Lock()
         self.__response_event = threading.Event()
         self.__current_response_message: Optional[codec.Message] = None
         self.__send_executor = ThreadPoolExecutor(
@@ -187,7 +191,7 @@ class _MessageSender(ResponseMessageListener):
         )
 
     def shutdown(self) -> None:
-        self.__send_executor.shutdown(wait=False, cancel_futures=False)
+        self.__send_executor.shutdown(wait=True, cancel_futures=False)
 
     def response_message_received(self, msg: codec.Message) -> None:
         """Invoked when response message is received by Message reader.
@@ -199,14 +203,14 @@ class _MessageSender(ResponseMessageListener):
         self.__response_event.set()
 
     def send_message(self, msg: codec.Message) -> None:
-        self.__send_async(msg, False)
+        self.__send_async(msg)
 
     def send_message_and_wait_for_response(
         self, msg: codec.Message, timeout: Optional[int] = 30
     ) -> Optional[codec.Message]:
         future = self.__send_async(msg, timeout)
         try:
-            return future.result(timeout)
+            return future.result(timeout + 1 if timeout else 1)
         except TimeoutError:
             logging.warning(
                 f"No response received for message within timeout: {msg}",
@@ -222,7 +226,7 @@ class _MessageSender(ResponseMessageListener):
     def __do_send(
         self, msg: codec.Message, wait_for_response_secs: Optional[int] = None
     ) -> Optional[codec.Message]:
-        with self.__send_lock:
+        with self._send_lock:
             if not self.__serial.is_open:
                 return None
             logging.debug(f"Sending message: {msg}, encoded: {msg.encode().hex()}")
@@ -232,7 +236,7 @@ class _MessageSender(ResponseMessageListener):
             except serial.SerialException as e:
                 logging.warning(f"Error sending message: {str(e)}", exc_info=False)
                 return None
-            if wait_for_response_secs:
+            if wait_for_response_secs and wait_for_response_secs > 0:
                 if self.__response_event.wait(wait_for_response_secs):
                     return self.__current_response_message
             return None
@@ -294,7 +298,6 @@ class _ReaderThread(threading.Thread):
         """Set reader in file mode and read file."""
         if hasattr(self.__serial, "timeout"):
             self.__serial.timeout = 10
-        self.__reset_file_mode()
         f = _FileDownload(
             file_size=size,
             file_timeout=timeout,
@@ -305,6 +308,9 @@ class _ReaderThread(threading.Thread):
         self.__f = f
         self.__file_mode = True
         try:
+            uart = codec.GetFileUart(types.File(original_file_name))
+            logging.debug(f"Sending message: {uart}, encoded: {uart.encode().hex()}")
+            self.__serial.write(uart.encode())
             if not self.__file_event.wait(timeout):
                 raise MissingResponseError("No file received within timeout")
             if self.__f.file_error:
@@ -332,13 +338,11 @@ class _ReaderThread(threading.Thread):
         self.alive = False
         if hasattr(self.__serial, "cancel_read"):
             self.__serial.cancel_read()
-        self.__message_listener_executor.shutdown(wait=False, cancel_futures=False)
+        self.__message_listener_executor.shutdown(wait=True, cancel_futures=False)
         self.__response_message_listener_executor.shutdown(
-            wait=False, cancel_futures=False
-        )
-        self.__file_download_listener_executor.shutdown(
-            wait=False, cancel_futures=False
+            wait=True, cancel_futures=False
         )
+        self.__file_download_listener_executor.shutdown(wait=True, cancel_futures=False)
         self.join(2)
 
     def run(self) -> None:
@@ -355,12 +359,23 @@ class _ReaderThread(threading.Thread):
                     self.__read_file(raw_header, self.__f)
                 else:
                     self.__read_protocol_message(raw_header)
-            except serial.SerialException:
+            except serial.SerialException as ser:
                 # probably some I/O problem such as disconnected USB serial adapters -> exit
-                logging.info("Serial port is closed (SerialException)")
+                logging.info(f"Serial port is closed (SerialException) {str(ser)}")
                 break
-            except OSError:
-                logging.info("OS Error reading from socket (OSError)")
+            except TypeError:
+                # read returned empty buffer
+                break
+            except OSError as ose:
+                logging.info(f"OS Error reading from socket (OSError): {str(ose)}")
+                break
+            except ValueError as ve:
+                logging.info(
+                    f"ValueError reading from socket (Probably disconnected): {str(ve)}"
+                )
+                break
+            except Exception as e:
+                logging.info(f"Exception reading from socket: {str(e)} - disconnecting")
                 break
         self.alive = False
         self.__notify_connection_listeners(connected=False)
@@ -374,7 +389,10 @@ class _ReaderThread(threading.Thread):
         loop_count = 0
         try:
             while remaining_size > 0 and self.__serial.is_open:
-                chunk = self.__serial.read(min(buffer_size, remaining_size))
+                bytes_to_read: Optional[int] = self.__serial.in_waiting
+                if not bytes_to_read or bytes_to_read <= 0:
+                    bytes_to_read = buffer_size
+                chunk = self.__serial.read(min(bytes_to_read, remaining_size))
                 if not chunk:
                     raise MissingResponseError("File download failed")
                 curr_pos = f.file_size - remaining_size
@@ -382,6 +400,8 @@ class _ReaderThread(threading.Thread):
                 in_memory_buffer[curr_pos : curr_pos + curr_len] = chunk
                 remaining_size -= curr_len
                 now = time.time()
+                if now == start:
+                    now += 1
                 if loop_count % 20 == 0:
                     self.__async_notify_file_download_in_progress(
                         f,
@@ -467,6 +487,8 @@ class _ReaderThread(threading.Thread):
         logging.debug(f"RECEIVE: Received header {raw_header.hex()}")
         msg_type, length = struct.unpack(">BH", raw_header)
         logging.debug(f"RECEIVE: Received msg type: {msg_type}, length: {length}")
+        if length > 20480:
+            raise ValueError(f"Message length too long: {length}")
         remaining_length = length - 3
         raw_message = raw_header
         while remaining_length > 0:
